@@ -1,49 +1,48 @@
 """
 
-    Abbreviations module for tokenization of Icelandic text
+Abbreviations module for tokenization of Icelandic text
 
-    Copyright (C) 2021 Miðeind ehf.
-    Original author: Vilhjálmur Þorsteinsson
+Copyright (C) 2016-2025 Miðeind ehf.
+Original author: Vilhjálmur Þorsteinsson
 
-    This software is licensed under the MIT License:
+This software is licensed under the MIT License:
 
-        Permission is hereby granted, free of charge, to any person
-        obtaining a copy of this software and associated documentation
-        files (the "Software"), to deal in the Software without restriction,
-        including without limitation the rights to use, copy, modify, merge,
-        publish, distribute, sublicense, and/or sell copies of the Software,
-        and to permit persons to whom the Software is furnished to do so,
-        subject to the following conditions:
+    Permission is hereby granted, free of charge, to any person
+    obtaining a copy of this software and associated documentation
+    files (the "Software"), to deal in the Software without restriction,
+    including without limitation the rights to use, copy, modify, merge,
+    publish, distribute, sublicense, and/or sell copies of the Software,
+    and to permit persons to whom the Software is furnished to do so,
+    subject to the following conditions:
 
-        The above copyright notice and this permission notice shall be
-        included in all copies or substantial portions of the Software.
+    The above copyright notice and this permission notice shall be
+    included in all copies or substantial portions of the Software.
 
-        THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-        EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-        MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-        IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-        CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-        TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-        SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-    This module reads the definition of abbreviations from the file
-    Abbrev.conf, assumed to be located in the same directory (or installation
-    resource library) as this Python source file.
+This module reads the definition of abbreviations from the file
+Abbrev.conf, assumed to be located in the same directory (or installation
+resource library) as this Python source file.
 
 """
 
-from typing import Generic, Iterator, Optional, Set, List, Dict, TypeVar
+from typing import Dict, Generic, Iterator, List, Optional, Set, TypeVar
 
 from threading import Lock
-from collections import defaultdict, OrderedDict
-from pkg_resources import resource_stream  # type: ignore
+from collections import defaultdict
+import importlib.resources as importlib_resources
 
 from .definitions import BIN_Tuple
 
 
 class ConfigError(Exception):
-
     pass
 
 
@@ -51,19 +50,22 @@ _T = TypeVar("_T")
 
 
 class OrderedSet(Generic[_T]):
-
-    """ Shim class to provide an ordered set API on top
-        of an OrderedDict. This is necessary to make abbreviation
-        lookups predictable and repeatable, which they would not be
-        if a standard Python set() was used. """
+    """Shim class to provide an ordered set API on top
+    of a dictionary. This is necessary to make abbreviation
+    lookups predictable and repeatable, which they would not be
+    if a standard Python set() was used."""
 
     def __init__(self) -> None:
-        self._dict: Dict[_T, None] = OrderedDict()
+        # Insertions are ordered in Python 3.7+ dicts
+        self._dict: Dict[_T, None] = {}
 
     def add(self, item: _T) -> None:
-        """ Add an item at the end of the ordered set """
-        if item not in self._dict:
-            self._dict[item] = None
+        """Add an item at the end of the ordered set"""
+        # For plain dicts in Python 3.7+, direct assignment works:
+        # * If item is new, it is added at the end.
+        # * If item already exists, its value is updated (to None again),
+        #   and the order remains unchanged.
+        self._dict[item] = None
 
     def __contains__(self, item: _T) -> bool:
         return item in self._dict
@@ -71,11 +73,13 @@ class OrderedSet(Generic[_T]):
     def __iter__(self) -> Iterator[_T]:
         return self._dict.__iter__()
 
+    def __repr__(self) -> str:
+        return f"{self.__class__.__name__}({list(self._dict.keys())})"
+
 
 class Abbreviations:
-
-    """ Wrapper around dictionary of abbreviations,
-        initialized from the config file """
+    """Wrapper around dictionary of abbreviations,
+    initialized from the config file"""
 
     # Dictionary of abbreviations and their meanings
     DICT: Dict[str, OrderedSet[BIN_Tuple]] = defaultdict(OrderedSet)
@@ -107,8 +111,8 @@ class Abbreviations:
 
     @staticmethod
     def add(abbrev: str, meaning: str, gender: str, fl: Optional[str] = None) -> None:
-        """ Add an abbreviation to the dictionary.
-            Called from the config file handler. """
+        """Add an abbreviation to the dictionary.
+        Called from the config file handler."""
         # Check for sentence finishers
         finisher = False
         not_finisher = False
@@ -137,7 +141,7 @@ class Abbreviations:
             # This logic is not fully present in Tokenizer as information
             # about person names is needed to make it work. The full implementation,
             # using the NAME_FINISHERS set, is found in bintokenizer.py in
-            # GreynirPackage.
+            # GreynirEngine.
             name_finisher = True
             abbrev = abbrev[0:-1]
             if not abbrev.endswith("."):
@@ -152,7 +156,14 @@ class Abbreviations:
         # Append the abbreviation and its meaning in tuple form
         # Multiple meanings are supported for each abbreviation
         Abbreviations.DICT[abbrev].add(
-            BIN_Tuple(meaning, 0, gender, "skst" if fl is None else fl, abbrev, "-",)
+            BIN_Tuple(
+                meaning,
+                0,
+                gender,
+                "skst" if fl is None else fl,
+                abbrev,
+                "-",
+            )
         )
         Abbreviations.MEANINGS.add(meaning)
         # Adding wrong versions of abbreviations
@@ -169,7 +180,14 @@ class Abbreviations:
                 # as abbreviations, even though they are listed as such
                 # in the form 'Í.' and 'Á.' for use within person names
                 Abbreviations.WRONGDICT[wabbrev].add(
-                    BIN_Tuple(meaning, 0, gender, "skst" if fl is None else fl, wabbrev, "-",)
+                    BIN_Tuple(
+                        meaning,
+                        0,
+                        gender,
+                        "skst" if fl is None else fl,
+                        wabbrev,
+                        "-",
+                    )
                 )
 
         elif "." in abbrev:
@@ -182,7 +200,14 @@ class Abbreviations:
                 wabbrev = abbrev[:i] + abbrev[i + 1 :]
                 Abbreviations.WRONGDOTS[wabbrev].append(abbrev)
                 Abbreviations.WRONGDICT[wabbrev].add(
-                    BIN_Tuple(meaning, 0, gender, "skst" if fl is None else fl, wabbrev, "-",)
+                    BIN_Tuple(
+                        meaning,
+                        0,
+                        gender,
+                        "skst" if fl is None else fl,
+                        wabbrev,
+                        "-",
+                    )
                 )
             if len(indices) > 2:
                 # 3 or 4 dots currently in vocabulary
@@ -214,7 +239,14 @@ class Abbreviations:
             Abbreviations.WRONGSINGLES.add(wabbrev)
             Abbreviations.WRONGDOTS[wabbrev].append(abbrev)
             Abbreviations.WRONGDICT[wabbrev].add(
-                BIN_Tuple(meaning, 0, gender, "skst" if fl is None else fl, wabbrev, "-",)
+                BIN_Tuple(
+                    meaning,
+                    0,
+                    gender,
+                    "skst" if fl is None else fl,
+                    wabbrev,
+                    "-",
+                )
             )
         if finisher:
             Abbreviations.FINISHERS.add(abbrev)
@@ -232,8 +264,8 @@ class Abbreviations:
         return meaning in Abbreviations.MEANINGS
 
     @staticmethod
-    def get_meaning(abbrev: str) -> Optional[List[BIN_Tuple]]:
-        """ Lookup meaning(s) of abbreviation, if available. """
+    def get_meaning(abbrev: str) -> Optional[list[BIN_Tuple]]:
+        """Look up meaning(s) of abbreviation, if available."""
         m = Abbreviations.DICT.get(abbrev)
         if not m:
             m = Abbreviations.WRONGDICT.get(abbrev)
@@ -241,7 +273,7 @@ class Abbreviations:
 
     @staticmethod
     def _handle_abbreviations(s: str) -> None:
-        """ Handle abbreviations in the settings section """
+        """Handle abbreviations in the settings section"""
         # Format: abbrev[*] = "meaning" gender (kk|kvk|hk)
         # An asterisk after an abbreviation ending with a period
         # indicates that the abbreviation may finish a sentence
@@ -272,44 +304,45 @@ class Abbreviations:
 
     @staticmethod
     def _handle_not_abbreviations(s: str) -> None:
-        """ Handle not_abbreviations in the settings section """
+        """Handle not_abbreviations in the settings section"""
         if len(s) < 3 or s[0] != '"' or s[-1] != '"':
             raise ConfigError("not_abbreviations should be enclosed in double quotes")
         Abbreviations.NOT_ABBREVIATIONS.add(s[1:-1])
 
     @staticmethod
-    def initialize():
-        """ Read the abbreviations config file """
+    def initialize() -> None:
+        """Read the abbreviations config file"""
         with Abbreviations._lock:
             if len(Abbreviations.DICT):
                 # Already initialized
                 return
 
             section = None
-            with resource_stream(__name__, "Abbrev.conf") as config:
-                for b in config:
-                    # We get lines as binary strings
-                    s = b.decode("utf-8")
-                    # Ignore comments
-                    ix = s.find("#")
-                    if ix >= 0:
-                        s = s[0:ix]
-                    s = s.strip()
-                    if not s:
-                        # Blank line: ignore
-                        continue
-                    if s[0] == "[":
-                        # Section header (we are expecting [abbreviations]/[not_abbreviations])
-                        if s not in {"[abbreviations]", "[not_abbreviations]"}:
-                            raise ConfigError("Wrong section header")
-                        section = s
-                        continue
-                    if section == "[abbreviations]":
-                        Abbreviations._handle_abbreviations(s)
-                    elif section == "[not_abbreviations]":
-                        Abbreviations._handle_not_abbreviations(s)
-                    else:
-                        raise ConfigError("Content outside section")
+
+            p = importlib_resources.files("tokenizer").joinpath("Abbrev.conf")
+            config = p.read_text(encoding="utf-8")
+
+            for s in config.split("\n"):
+                # Ignore comments
+                ix = s.find("#")
+                if ix >= 0:
+                    s = s[0:ix]
+                s = s.strip()
+                if not s:
+                    # Blank line: ignore
+                    continue
+                if s[0] == "[":
+                    # Section header (we expect [abbreviations]/[not_abbreviations])
+                    if s not in {"[abbreviations]", "[not_abbreviations]"}:
+                        raise ConfigError("Wrong section header")
+                    section = s
+                    continue
+                if section == "[abbreviations]":
+                    Abbreviations._handle_abbreviations(s)
+                elif section == "[not_abbreviations]":
+                    Abbreviations._handle_not_abbreviations(s)
+                else:
+                    raise ConfigError("Content outside section")
 
             # Remove not_abbreviations from WRONGDICT
             for abbr in Abbreviations.NOT_ABBREVIATIONS:
