@@ -1,43 +1,42 @@
 """
 
-    Definitions used for tokenization of Icelandic text
+Definitions used for tokenization of Icelandic text
 
-    Copyright (C) 2021 Miðeind ehf.
-    Original author: Vilhjálmur Þorsteinsson
+Copyright (C) 2016-2025 Miðeind ehf.
+Original author: Vilhjálmur Þorsteinsson
 
-    This software is licensed under the MIT License:
+This software is licensed under the MIT License:
 
-        Permission is hereby granted, free of charge, to any person
-        obtaining a copy of this software and associated documentation
-        files (the "Software"), to deal in the Software without restriction,
-        including without limitation the rights to use, copy, modify, merge,
-        publish, distribute, sublicense, and/or sell copies of the Software,
-        and to permit persons to whom the Software is furnished to do so,
-        subject to the following conditions:
+    Permission is hereby granted, free of charge, to any person
+    obtaining a copy of this software and associated documentation
+    files (the "Software"), to deal in the Software without restriction,
+    including without limitation the rights to use, copy, modify, merge,
+    publish, distribute, sublicense, and/or sell copies of the Software,
+    and to permit persons to whom the Software is furnished to do so,
+    subject to the following conditions:
 
-        The above copyright notice and this permission notice shall be
-        included in all copies or substantial portions of the Software.
+    The above copyright notice and this permission notice shall be
+    included in all copies or substantial portions of the Software.
 
-        THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-        EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-        MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-        IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-        CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-        TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-        SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 """
 
 from typing import (
-    Dict,
-    FrozenSet,
-    Tuple,
+    Mapping,
     Union,
     Callable,
-    List,
     Sequence,
     Optional,
     NamedTuple,
+    Tuple,
+    List,
     cast,
 )
 
@@ -54,27 +53,33 @@ AmountTuple = Tuple[float, str, Optional[List[str]], Optional[List[str]]]
 TelnoTuple = Tuple[str, str]
 CurrencyTuple = Tuple[str, Optional[List[str]], Optional[List[str]]]
 
-BIN_Tuple = NamedTuple(
-    "BIN_Tuple",
-    [
-        ("stofn", str),
-        ("utg", int),
-        ("ordfl", str),
-        ("fl", str),
-        ("ordmynd", str),
-        ("beyging", str),
-    ],
-)
+
+class BIN_Tuple(NamedTuple):
+    stofn: str
+    utg: int
+    ordfl: str
+    fl: str
+    ordmynd: str
+    beyging: str
+
+    def __str__(self) -> str:
+        """Return string representation as a plain tuple (a, b, c, ...)"""
+        return str(tuple(self))
+
+    def __repr__(self) -> str:
+        """Return repr as a plain tuple for consistency with README examples"""
+        return repr(tuple(self))
+
+
 BIN_TupleList = Sequence[BIN_Tuple]
 
-PersonNameTuple = NamedTuple(
-    "PersonNameTuple",
-    [
-        ("name", str),
-        ("gender", Optional[str]),
-        ("case", Optional[str]),
-    ]
-)
+
+class PersonNameTuple(NamedTuple):
+    name: str
+    gender: Optional[str]
+    case: Optional[str]
+
+
 PersonNameList = Sequence[PersonNameTuple]
 
 # All possible contents of the Tok.val attribute
@@ -105,7 +110,8 @@ ZEROWIDTH_SPACE = chr(8203)
 ZEROWIDTH_NBSP = chr(65279)
 
 # Preprocessing of unicode characters before tokenization
-UNICODE_REPLACEMENTS = {
+# Composite glyph replacements - only combining accents and umlauts
+COMPOSITE_REPLACEMENTS: Mapping[str, str] = {
     # Translations of separate umlauts and accents to single glyphs.
     # The strings to the left in each tuple are two Unicode code
     # points: vowel + COMBINING ACUTE ACCENT (chr(769)) or
@@ -130,17 +136,39 @@ UNICODE_REPLACEMENTS = {
     "U" + UMLAUT: "Ü",
     "Y" + ACCENT: "Ý",
     "O" + UMLAUT: "Ö",
-    # Also remove these unwanted characters
+}
+
+# Zero-width characters that should always be removed
+ZEROWIDTH_CHARACTERS: Mapping[str, str] = {
     SOFT_HYPHEN: "",
     ZEROWIDTH_SPACE: "",
     ZEROWIDTH_NBSP: "",
 }
+
+# Combined dictionary for backward compatibility
+UNICODE_REPLACEMENTS: Mapping[str, str] = {**COMPOSITE_REPLACEMENTS, **ZEROWIDTH_CHARACTERS}
+
+# Regex for composite glyphs only
+COMPOSITE_REGEX = re.compile(
+    r"|".join(map(_escape, COMPOSITE_REPLACEMENTS.keys())), re.UNICODE
+)
+
+# Regex for zero-width characters only
+ZEROWIDTH_REGEX = re.compile(
+    r"|".join(map(_escape, ZEROWIDTH_CHARACTERS.keys())), re.UNICODE
+)
+
+# Combined regex for backward compatibility
 UNICODE_REGEX = re.compile(
     r"|".join(map(_escape, UNICODE_REPLACEMENTS.keys())), re.UNICODE
 )
 
 # Used for the first step of token splitting
 ROUGH_TOKEN_REGEX = re.compile(r"(\s*)([^\s]*)", re.UNICODE)
+# Constants for readability when using the ROUGH_TOKEN_REGEX
+ROUGH_TOKEN_REGEX_ENTIRE_MATCH = 0
+ROUGH_TOKEN_REGEX_WHITE_SPACE_GROUP = 1
+ROUGH_TOKEN_REGEX_TOKEN_GROUP = 2
 
 # Hyphens are normalized to '-'
 HYPHEN = "-"  # Normal hyphen
@@ -156,8 +184,8 @@ COMPOSITE_HYPHEN = EN_DASH
 # Recognized punctuation
 LEFT_PUNCTUATION = "([„‚«#$€£¥₽<"
 RIGHT_PUNCTUATION = ".,:;)]!%‰?“»”’‛‘…>°"
-CENTER_PUNCTUATION = '"*•&+=@©|'
-NONE_PUNCTUATION = "^/±'´~\\" + HYPHEN + EN_DASH + EM_DASH
+CENTER_PUNCTUATION = '"*•&+=@©|' + EM_DASH
+NONE_PUNCTUATION = "^/±'´~\\" + HYPHEN + EN_DASH
 PUNCTUATION = (
     LEFT_PUNCTUATION + CENTER_PUNCTUATION + RIGHT_PUNCTUATION + NONE_PUNCTUATION
 )
@@ -199,6 +227,9 @@ PUNCT_INSIDE_WORD = frozenset([".", "'", "‘", "´", "’", HYPHEN, EN_DASH])
 PUNCT_ENDING_WORD = frozenset(["'", "²", "³"])
 # Punctuation symbols that may occur together
 PUNCT_COMBINATIONS = frozenset(["?", "!", "…"])
+# Punctuation in end of indirect speech that doesn't necessarily end sentences
+PUNCT_INDIRECT_SPEECH = frozenset(["?", "!"])
+
 
 # Single and double quotes
 SQUOTES = "'‚‛‘´"
@@ -210,17 +241,23 @@ CLOCK_ABBREVS = frozenset(("kl", "kl.", "klukkan"))
 TELNO_PREFIXES = "45678"
 
 # Known telephone country codes
-COUNTRY_CODES = frozenset(("354", "+354", "00354",))
+COUNTRY_CODES = frozenset(
+    (
+        "354",
+        "+354",
+        "00354",
+    )
+)
 
 # Words that can precede a year number; will be assimilated into the year token
 YEAR_WORD = frozenset(("árið", "ársins", "árinu"))
 
 # Characters that can start a numeric token
 DIGITS_PREFIX = frozenset([d for d in "0123456789"])
-SIGN_PREFIX = frozenset(("+", "-"))
+SIGN_PREFIX = frozenset(("+", HYPHEN))
 
 # Month names and numbers
-MONTHS = {
+MONTHS: Mapping[str, int] = {
     "janúar": 1,
     "janúars": 1,
     "febrúar": 2,
@@ -330,31 +367,31 @@ DAYS_IN_MONTH = (0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31)
 # }
 
 # Time of day expressions spelled out
-CLOCK_NUMBERS = {
-    "eitt": [1, 0, 0],
-    "tvö": [2, 0, 0],
-    "þrjú": [3, 0, 0],
-    "fjögur": [4, 0, 0],
-    "fimm": [5, 0, 0],
-    "sex": [6, 0, 0],
-    "sjö": [7, 0, 0],
-    "átta": [8, 0, 0],
-    "níu": [9, 0, 0],
-    "tíu": [10, 0, 0],
-    "ellefu": [11, 0, 0],
-    "tólf": [12, 0, 0],
-    "hálfeitt": [12, 30, 0],
-    "hálftvö": [1, 30, 0],
-    "hálfþrjú": [2, 30, 0],
-    "hálffjögur": [3, 30, 0],
-    "hálffimm": [4, 30, 0],
-    "hálfsex": [5, 30, 0],
-    "hálfsjö": [6, 30, 0],
-    "hálfátta": [7, 30, 0],
-    "hálfníu": [8, 30, 0],
-    "hálftíu": [9, 30, 0],
-    "hálfellefu": [10, 30, 0],
-    "hálftólf": [11, 30, 0],
+CLOCK_NUMBERS: Mapping[str, tuple[int, int, int]] = {
+    "eitt": (1, 0, 0),
+    "tvö": (2, 0, 0),
+    "þrjú": (3, 0, 0),
+    "fjögur": (4, 0, 0),
+    "fimm": (5, 0, 0),
+    "sex": (6, 0, 0),
+    "sjö": (7, 0, 0),
+    "átta": (8, 0, 0),
+    "níu": (9, 0, 0),
+    "tíu": (10, 0, 0),
+    "ellefu": (11, 0, 0),
+    "tólf": (12, 0, 0),
+    "hálfeitt": (12, 30, 0),
+    "hálftvö": (1, 30, 0),
+    "hálfþrjú": (2, 30, 0),
+    "hálffjögur": (3, 30, 0),
+    "hálffimm": (4, 30, 0),
+    "hálfsex": (5, 30, 0),
+    "hálfsjö": (6, 30, 0),
+    "hálfátta": (7, 30, 0),
+    "hálfníu": (8, 30, 0),
+    "hálftíu": (9, 30, 0),
+    "hálfellefu": (10, 30, 0),
+    "hálftólf": (11, 30, 0),
 }
 
 # Set of words only possible in temporal phrases
@@ -410,7 +447,7 @@ CURRENCY_ABBREV = frozenset(
 )
 
 # Map symbols to currency abbreviations
-CURRENCY_SYMBOLS = {
+CURRENCY_SYMBOLS: Mapping[str, str] = {
     "$": "USD",
     "€": "EUR",
     "£": "GBP",
@@ -422,7 +459,7 @@ CURRENCY_SYMBOLS = {
 SINGLECHAR_FRACTIONS = "↉⅒⅑⅛⅐⅙⅕¼⅓½⅖⅔⅜⅗¾⅘⅝⅚⅞"
 
 # Derived unit : (base SI unit, conversion factor/function)
-SI_UNITS: Dict[str, Tuple[str, Union[float, Callable[[float], float]]]] = {
+SI_UNITS: Mapping[str, tuple[str, Union[float, Callable[[float], float]]]] = {
     # Distance
     "m": ("m", 1.0),
     "mm": ("m", 1.0e-3),
@@ -518,15 +555,15 @@ SI_UNITS: Dict[str, Tuple[str, Union[float, Callable[[float], float]]]] = {
     # "km/klst.": ("m/s", 1000.0/(60*60)),
 }
 
-DIRECTIONS = {
+DIRECTIONS: Mapping[str, str] = {
     "N": "Norður",
 }
 
-_unit_lambda: Callable[[str], str] = lambda unit: unit + r"(?!\w)" if unit[
-    -1
-].isalpha() else unit
+_unit_lambda: Callable[[str], str] = lambda unit: (
+    unit + r"(?!\w)" if unit[-1].isalpha() else unit
+)
 
-SI_UNITS_SET: FrozenSet[str] = frozenset(SI_UNITS.keys())
+SI_UNITS_SET: frozenset[str] = frozenset(SI_UNITS.keys())
 SI_UNITS_REGEX_STRING = r"|".join(
     map(
         # If the unit ends with a letter, don't allow the next character
@@ -582,7 +619,7 @@ KLUDGY_ORDINALS_MODIFY = 1
 KLUDGY_ORDINALS_TRANSLATE = 2
 
 # Incorrectly written ('kludgy') ordinals
-ORDINAL_ERRORS = {
+ORDINAL_ERRORS: Mapping[str, str] = {
     "1sti": "fyrsti",
     "1sta": "fyrsta",
     "1stu": "fyrstu",
@@ -603,7 +640,7 @@ ORDINAL_ERRORS = {
 }
 
 # Translations of kludgy ordinal words into numbers
-ORDINAL_NUMBERS = {
+ORDINAL_NUMBERS: Mapping[str, int] = {
     "1sti": 1,
     "1sta": 1,
     "1stu": 1,
@@ -633,7 +670,7 @@ ROMAN_NUMERAL_MAP = tuple(
 
 
 def roman_to_int(s: str) -> int:
-    """ Quick and dirty conversion of an already validated Roman numeral to integer """
+    """Quick and dirty conversion of an already validated Roman numeral to integer"""
     # Adapted from http://code.activestate.com/recipes/81611-roman-numerals/
     i = result = 0
     for integer, numeral in ROMAN_NUMERAL_MAP:
@@ -644,60 +681,19 @@ def roman_to_int(s: str) -> int:
     return result
 
 
-# Recognize words that multiply numbers
-MULTIPLIERS = {
-    # "núll": 0,
-    # "hálfur": 0.5,
-    # "helmingur": 0.5,
-    # "þriðjungur": 1.0 / 3,
-    # "fjórðungur": 1.0 / 4,
-    # "fimmtungur": 1.0 / 5,
-    "einn": 1,
-    "tveir": 2,
-    "þrír": 3,
-    "fjórir": 4,
-    "fimm": 5,
-    "sex": 6,
-    "sjö": 7,
-    "átta": 8,
-    "níu": 9,
-    "tíu": 10,
-    "ellefu": 11,
-    "tólf": 12,
-    "þrettán": 13,
-    "fjórtán": 14,
-    "fimmtán": 15,
-    "sextán": 16,
-    "sautján": 17,
-    "seytján": 17,
-    "átján": 18,
-    "nítján": 19,
-    "tuttugu": 20,
-    "þrjátíu": 30,
-    "fjörutíu": 40,
-    "fimmtíu": 50,
-    "sextíu": 60,
-    "sjötíu": 70,
-    "áttatíu": 80,
-    "níutíu": 90,
-    # "par": 2,
-    # "tugur": 10,
-    # "tylft": 12,
-    "hundrað": 100,
-    "þúsund": 1000,  # !!! Bæði hk og kvk!
+NUMBER_ABBREV: Mapping[str, int] = {
     "þús.": 1000,
-    "milljón": 1e6,
-    "milla": 1e6,
-    "millj.": 1e6,
-    "mljó.": 1e6,
-    "milljarður": 1e9,
-    "miljarður": 1e9,
-    "ma.": 1e9,
-    "mrð.": 1e9,
+    "millj.": 10**6,
+    "mljó.": 10**6,
+    "ma.": 10**9,
+    "mrð.": 10**9,
+    "billj.": 10**12,
+    "bljó.": 10**12,
+    "trillj.": 10**18,
 }
 
 # Recognize words for percentages
-PERCENTAGES = {
+PERCENTAGES: Mapping[str, int] = {
     "prósent": 1,
     "prósenta": 1,
     "prósenti": 1,
@@ -717,7 +713,7 @@ PERCENTAGES = {
 
 # Amount abbreviations including 'kr' for the ISK
 # Corresponding abbreviations are found in Abbrev.conf
-AMOUNT_ABBREV = {
+AMOUNT_ABBREV: Mapping[str, Union[int, float]] = {
     "kr.": 1,
     "kr": 1,
     "krónur": 1,
@@ -739,14 +735,19 @@ AMOUNT_ABBREV = {
     "mö.kr": 1e9,
     "mlja.kr.": 1e9,
     "mlja.kr": 1e9,
+    # "mrð.kr.": 1e9,
+    # "mrð.kr": 1e9,
+    # "billj.kr.": 1e12,
+    # "billj.kr": 1e12,
+    # "trillj.kr.": 1e18,
+    # "trillj.kr": 1e18,
 }
 
 # Króna amount strings allowed before a number, e.g. "kr. 9.900"
 ISK_AMOUNT_PRECEDING = frozenset(("kr.", "kr", "krónur"))
 
-# URL prefixes. Note that this list should not contain www since
-# www.something.com is a domain token, not a URL token.
-URL_PREFIXES = (
+# URI scheme prefixes
+URI_PREFIXES = (
     "http://",
     "https://",
     "file://",
@@ -754,6 +755,27 @@ URL_PREFIXES = (
     "ssh://",
     "sftp://",
     "smb://",
+    "git://",
+    "git+ssh://",
+    "svn://",
+    "svn+ssh://",
+    "imap://",
+    "rtmp://",
+    "rtsp://",
+    "telnet://",
+    "udp://",
+    "vnc://",
+    "irc://",
+    "nntp://",
+    "wss://",
+    "ws://",
+    "xmpp://",
+    "mtqp://",
+    "afp://",
+    "nfs://",
+    "mms://",
+    "tftp://",
+    "ldap://",
 )
 
 TOP_LEVEL_DOMAINS = frozenset(
@@ -1189,8 +1211,8 @@ KT_MAGIC = [3, 2, 7, 6, 5, 4, 0, 3, 2]
 
 
 def valid_ssn(kt: str) -> bool:
-    """ Validate Icelandic social security number """
-    if not kt or len(kt) != 11 or kt[6] != "-":
+    """Validate Icelandic social security number ("kennitala")"""
+    if not kt or len(kt) != 11 or (kt[6] != HYPHEN and kt[6] != EN_DASH):
         return False
     m = 11 - sum((ord(kt[i]) - 48) * KT_MAGIC[i] for i in range(9)) % 11
     c = ord(kt[9]) - 48
@@ -1200,7 +1222,7 @@ def valid_ssn(kt: str) -> bool:
 # HTML escaped characters/ligatures, e.g. '&aacute;' meaning 'á'.
 # The following is a subset of HTML escape codes, roughly selected
 # by analyzing the content of the Icelandic Gigaword Corpus (Risamálheild).
-HTML_ESCAPES = {
+HTML_ESCAPES: Mapping[str, str] = {
     # Icelandic letters
     "aacute": "á",
     "eth": "ð",
@@ -1252,8 +1274,8 @@ HTML_ESCAPES = {
     "emsp": " ",
     "thinsp": " ",
     # Dashes and hyphens
-    "ndash": "–",
-    "mdash": "—",
+    "ndash": EN_DASH,
+    "mdash": EM_DASH,
     # The soft hyphen &shy; is mapped to an empty string
     "shy": "",
     # Other non-ASCII letters
