@@ -1,41 +1,52 @@
 #!/usr/bin/env python
 """
 
-    Tokenizer for Icelandic text
+Tokenizer for Icelandic text
 
-    Copyright (C) 2021 Miðeind ehf.
-    Original author: Vilhjálmur Þorsteinsson
+Copyright (C) 2016-2025 Miðeind ehf.
+Original author: Vilhjálmur Þorsteinsson
 
-    This software is licensed under the MIT License:
+This software is licensed under the MIT License:
 
-        Permission is hereby granted, free of charge, to any person
-        obtaining a copy of this software and associated documentation
-        files (the "Software"), to deal in the Software without restriction,
-        including without limitation the rights to use, copy, modify, merge,
-        publish, distribute, sublicense, and/or sell copies of the Software,
-        and to permit persons to whom the Software is furnished to do so,
-        subject to the following conditions:
+    Permission is hereby granted, free of charge, to any person
+    obtaining a copy of this software and associated documentation
+    files (the "Software"), to deal in the Software without restriction,
+    including without limitation the rights to use, copy, modify, merge,
+    publish, distribute, sublicense, and/or sell copies of the Software,
+    and to permit persons to whom the Software is furnished to do so,
+    subject to the following conditions:
 
-        The above copyright notice and this permission notice shall be
-        included in all copies or substantial portions of the Software.
+    The above copyright notice and this permission notice shall be
+    included in all copies or substantial portions of the Software.
 
-        THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-        EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-        MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-        IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-        CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-        TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-        SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-    This is an executable program wrapper (main module) for the Tokenizer
-    package. It can be used to invoke the Tokenizer from the command line,
-    or via fork() or exec(), with the command 'tokenize'. The main() function
-    of this module is registered as a console_script entry point in setup.py.
+This is an executable program wrapper (main module) for the Tokenizer
+package. It can be used to invoke the Tokenizer from the command line,
+or via fork() or exec(), with the command 'tokenize'. The main() function
+of this module is registered as a CLI command in pyproject.toml.
 
 """
 
-from typing import TextIO, Dict, Iterator, List, Callable, Any, Tuple, Union, cast
+from typing import (
+    Dict,
+    Iterable,
+    List,
+    TextIO,
+    Iterator,
+    Callable,
+    Any,
+    Tuple,
+    Union,
+    cast,
+)
 
 import sys
 import argparse
@@ -44,11 +55,9 @@ from functools import partial
 
 from .definitions import AmountTuple, BIN_Tuple, NumberTuple, PunctuationTuple
 from .tokenizer import TOK, Tok, tokenize
+from . import __version__ as tokenizer_version
 
 
-ReadFile = argparse.FileType("r", encoding="utf-8")
-WriteFile = argparse.FileType("w", encoding="utf-8")
-
 # Define the command line arguments
 
 parser = argparse.ArgumentParser(description="Tokenizes Icelandic text")
@@ -56,7 +65,7 @@ parser = argparse.ArgumentParser(description="Tokenizes Icelandic text")
 parser.add_argument(
     "infile",
     nargs="?",
-    type=ReadFile,
+    type=argparse.FileType("r", encoding="utf-8"),
     default=sys.stdin,
     help="UTF-8 text file to tokenize",
 )
@@ -64,7 +73,7 @@ parser.add_argument(
 parser.add_argument(
     "outfile",
     nargs="?",
-    type=WriteFile,
+    type=argparse.FileType("w", encoding="utf-8"),
     default=sys.stdout,
     help="UTF-8 output text file",
 )
@@ -110,7 +119,10 @@ parser.add_argument(
 )
 
 parser.add_argument(
-    "-o", "--original", action="store_true", help="Outputs original text of tokens",
+    "-o",
+    "--original",
+    action="store_true",
+    help="Outputs original text of tokens",
 )
 
 parser.add_argument(
@@ -150,29 +162,37 @@ parser.add_argument(
     ),
 )
 
+parser.add_argument(
+    "-v",
+    "--version",
+    action="version",
+    version=f"%(prog)s {tokenizer_version}",
+    help="Show the version number and exit",
+)
+
 
 def main() -> None:
-    """ Main function, called when the tokenize command is invoked """
+    """Main function, called when the tokenize command is invoked"""
 
     args = parser.parse_args()
-    options: Dict[str, bool] = dict()
+    options: Dict[str, bool] = {}
 
     def quote(s: str) -> str:
-        """ Return the string s within double quotes, and with any contained
-            backslashes and double quotes escaped with a backslash """
+        """Return the string s within double quotes, and with any contained
+        backslashes and double quotes escaped with a backslash"""
         return '"' + s.replace("\\", "\\\\").replace('"', '\\"') + '"'
 
-    def spanquote(l: List[int]) -> str:
-        """ Return the list l as a string within double quotes """
-        return '"' + "-".join(str(x) for x in l) + '"'
+    def spanquote(lst: Iterable[int]) -> str:
+        """Return the list lst as a string within double quotes"""
+        return '"' + "-".join(str(x) for x in lst) + '"'
 
     def gen(f: TextIO) -> Iterator[str]:
-        """ Generate the lines of text in the input file """
+        """Generate the lines of text in the input file"""
         for line in f:
             yield line
 
     def val(t: Tok, quote_word: bool = False) -> Any:
-        """ Return the value part of the token t """
+        """Return the value part of the token t"""
         if t.val is None:
             return None
         if t.kind == TOK.WORD:
@@ -252,8 +272,7 @@ def main() -> None:
     # Configure our JSON dump function
     json_dumps = partial(json.dumps, ensure_ascii=False, separators=(",", ":"))
     curr_sent: List[str] = []
-    sep = "" if args.original else " "
-
+    tsep = "" if args.original else " "  # token separator
     for t in tokenize(gen(args.infile), **options):
         if args.csv:
             # Output the tokens in CSV format, one line per token
@@ -273,8 +292,8 @@ def main() -> None:
                 print('0,"","","",""', file=args.outfile)
         elif args.json:
             # Output the tokens in JSON format, one line per token
-            d: Dict[str, Union[str, List[int]]] = dict(k=TOK.descr[t.kind])
-            if t.txt is not None:
+            d: Dict[str, Union[str, List[int]]] = {"k": TOK.descr[t.kind]}
+            if t.txt:
                 d["t"] = t.txt
             v = val(t)
             if v is not None:
@@ -285,20 +304,18 @@ def main() -> None:
                 d["s"] = t.origin_spans
             print(json_dumps(d), file=args.outfile)
         else:
-            # Normal shallow parse, one line per sentence,
+            # Normal shallow parse, sentences separated by newline by default,
             # tokens separated by spaces
             if t.kind in TOK.END:
                 # End of sentence/paragraph
                 if curr_sent:
-                    print(sep.join(curr_sent), file=args.outfile)
+                    print(tsep.join(curr_sent), file=args.outfile)
                     curr_sent = []
-            else:
-                txt = to_text(t)
-                if txt:
-                    curr_sent.append(txt)
-
+            txt = to_text(t)
+            if txt:
+                curr_sent.append(txt)
     if curr_sent:
-        print(sep.join(curr_sent), file=args.outfile)
+        print(tsep.join(curr_sent), file=args.outfile)
 
 
 if __name__ == "__main__":
