@@ -4,7 +4,7 @@
 
     Tests for Tokenizer module
 
-    Copyright (C) 2021 by Miðeind ehf.
+    Copyright (C) 2016-2025 by Miðeind ehf.
 
     This software is licensed under the MIT License:
 
@@ -167,7 +167,6 @@ def test_small_difficult_cases() -> None:
         assert char_indexes == [0, 2, 4]
         assert byte_indexes == [0, 2, 4]
 
-
     # Two byte characters
     for x in ["þ", "æ", "á"]:
         s = x
@@ -228,12 +227,11 @@ def test_small_difficult_cases() -> None:
         assert char_indexes == [0, 2, 4]
         assert byte_indexes == [0, 3, 6]
 
-
     # Two character characters
     # These strings contain two unicode code points that are rendered as one letter.
     # They are counted as two characters in python.
     # In addition the accent and umlaut characters are two bytes.
-    for x in ["a"+ACCENT, "o"+UMLAUT]:
+    for x in ["a" + ACCENT, "o" + UMLAUT]:
         s = x
         toks = tokenizer.parse_tokens([s])
         char_indexes, byte_indexes = tokenizer.calculate_indexes(toks)
@@ -286,11 +284,11 @@ def test_small_difficult_cases() -> None:
         # example chars:
         #   " a´ a´"
         #    012345
-        #    ^  ^    
+        #    ^  ^
         # example bytes:
         #   " a´_ a´_"
         #    01234567
-        #    ^   ^  
+        #    ^   ^
         toks = tokenizer.parse_tokens([s])
         char_indexes, byte_indexes = tokenizer.calculate_indexes(toks)
         assert char_indexes == [0, 3]
@@ -300,7 +298,6 @@ def test_small_difficult_cases() -> None:
         assert char_indexes == [0, 3, 6]
         assert byte_indexes == [0, 4, 8]
 
-
     # The em-dash is 3 bytes
     for x in [EM_DASH]:
         s = x
@@ -359,7 +356,7 @@ def test_small_difficult_cases() -> None:
         # example bytes:
         #   " a__ a__"
         #    01234567
-        #    ^   ^  
+        #    ^   ^
         toks = tokenizer.parse_tokens([s])
         char_indexes, byte_indexes = tokenizer.calculate_indexes(toks)
         assert char_indexes == [0, 2]
@@ -377,25 +374,181 @@ def test_larger_case() -> None:
     #    x                x                     x  xx                   x
     toks = tokenizer.parse_tokens([s])
     char_indexes, byte_indexes = tokenizer.calculate_indexes(toks)
-    assert char_indexes == [0, 5, 13, 16, 18, 25, 30, 33, 36, 40, 45, 50, 53, 61, 66, 72]
-    assert byte_indexes == [0, 6, 14, 17, 20, 27, 32, 35, 38, 43, 50, 55, 58, 66, 72, 78]
+    assert char_indexes == [
+        0,
+        5,
+        13,
+        16,
+        18,
+        25,
+        30,
+        33,
+        36,
+        40,
+        45,
+        50,
+        53,
+        61,
+        66,
+        72,
+    ]
+    assert byte_indexes == [
+        0,
+        6,
+        14,
+        17,
+        20,
+        27,
+        32,
+        35,
+        38,
+        43,
+        50,
+        55,
+        58,
+        66,
+        72,
+        78,
+    ]
     toks = tokenizer.parse_tokens([s])
     char_indexes, byte_indexes = tokenizer.calculate_indexes(toks, last_is_end=True)
-    assert char_indexes == [0, 5, 13, 16, 18, 25, 30, 33, 36, 40, 45, 50, 53, 61, 66, 72, 73]
-    assert byte_indexes == [0, 6, 14, 17, 20, 27, 32, 35, 38, 43, 50, 55, 58, 66, 72, 78, 79]
+    assert char_indexes == [
+        0,
+        5,
+        13,
+        16,
+        18,
+        25,
+        30,
+        33,
+        36,
+        40,
+        45,
+        50,
+        53,
+        61,
+        66,
+        72,
+        73,
+    ]
+    assert byte_indexes == [
+        0,
+        6,
+        14,
+        17,
+        20,
+        27,
+        32,
+        35,
+        38,
+        43,
+        50,
+        55,
+        58,
+        66,
+        72,
+        78,
+        79,
+    ]
 
 
 def test_iterator_cases() -> None:
-    s = ["Þessi ", "setning ", "er ", "í ", "lengra ", "lagi ", "og ", "er ", "með ", "bæði ", "eins ", "og ", "tveggja ", "bæta ", "stafi."]
+    s = [
+        "Þessi ",
+        "setning ",
+        "er ",
+        "í ",
+        "lengra ",
+        "lagi ",
+        "og ",
+        "er ",
+        "með ",
+        "bæði ",
+        "eins ",
+        "og ",
+        "tveggja ",
+        "bæta ",
+        "stafi.",
+    ]
     # (char and byte indexes in a similar test above)
     toks = tokenizer.parse_tokens(s)
     char_indexes, byte_indexes = tokenizer.calculate_indexes(toks)
-    assert char_indexes == [0, 5, 13, 16, 18, 25, 30, 33, 36, 40, 45, 50, 53, 61, 66, 72]
-    assert byte_indexes == [0, 6, 14, 17, 20, 27, 32, 35, 38, 43, 50, 55, 58, 66, 72, 78]
+    assert char_indexes == [
+        0,
+        5,
+        13,
+        16,
+        18,
+        25,
+        30,
+        33,
+        36,
+        40,
+        45,
+        50,
+        53,
+        61,
+        66,
+        72,
+    ]
+    assert byte_indexes == [
+        0,
+        6,
+        14,
+        17,
+        20,
+        27,
+        32,
+        35,
+        38,
+        43,
+        50,
+        55,
+        58,
+        66,
+        72,
+        78,
+    ]
     toks = tokenizer.parse_tokens(s)
     char_indexes, byte_indexes = tokenizer.calculate_indexes(toks, last_is_end=True)
-    assert char_indexes == [0, 5, 13, 16, 18, 25, 30, 33, 36, 40, 45, 50, 53, 61, 66, 72, 73]
-    assert byte_indexes == [0, 6, 14, 17, 20, 27, 32, 35, 38, 43, 50, 55, 58, 66, 72, 78, 79]
+    assert char_indexes == [
+        0,
+        5,
+        13,
+        16,
+        18,
+        25,
+        30,
+        33,
+        36,
+        40,
+        45,
+        50,
+        53,
+        61,
+        66,
+        72,
+        73,
+    ]
+    assert byte_indexes == [
+        0,
+        6,
+        14,
+        17,
+        20,
+        27,
+        32,
+        35,
+        38,
+        43,
+        50,
+        55,
+        58,
+        66,
+        72,
+        78,
+        79,
+    ]
 
     s = ["Stutt setning.", "", "Önnur setning."]
     #     01234567890123        45678901234567
@@ -426,34 +579,34 @@ def test_iterator_cases() -> None:
 
 
 def test_paragraph_markers() -> None:
-    s = " [[ Stutt setning. ]] [[ ]] [[ Önnur setning. ]]"
+    s = "[[Stutt setning.]][[]][[Önnur setning.]]"
     #    012345678901234567890123456789012345678901234567
-    #    ^  ^     ^       ^^  ^  ^  ^  ^     ^       ^^
+    #    ^^^    ^       ^^ ^ ^ ^ ^    ^       ^^
     #                                   x
     toks = tokenizer.parse_tokens(s)
     char_indexes, byte_indexes = tokenizer.calculate_indexes(toks)
-    assert char_indexes == [0, 3, 9, 17, 18, 21, 24, 27, 30, 36, 44, 45]
-    assert byte_indexes == [0, 3, 9, 17, 18, 21, 24, 27, 30, 37, 45, 46]
+    assert char_indexes == [0, 2, 7, 15, 16, 18, 20, 22, 24, 29, 37, 38]
+    assert byte_indexes == [0, 2, 7, 15, 16, 18, 20, 22, 24, 30, 38, 39]
     toks = tokenizer.parse_tokens(s)
     char_indexes, byte_indexes = tokenizer.calculate_indexes(toks, last_is_end=True)
-    assert char_indexes == [0, 3, 9, 17, 18, 21, 24, 27, 30, 36, 44, 45, 48]
-    assert byte_indexes == [0, 3, 9, 17, 18, 21, 24, 27, 30, 37, 45, 46, 49]
+    assert char_indexes == [0, 2, 7, 15, 16, 18, 20, 22, 24, 29, 37, 38, 40]
+    assert byte_indexes == [0, 2, 7, 15, 16, 18, 20, 22, 24, 30, 38, 39, 41]
 
     # The tokenize functions does stuff to paragraph markers. Test that the
     # indexes are properly calculated after that.
     # Note that the text of the dropped empty paragraph markers disappears.
-    s = " [[ Stutt setning. ]] [[ ]] [[ Önnur setning. ]]"
+    s = "[[Stutt setning.]][[]][[Önnur setning.]]"
     #    012345678901234567890123456789012345678901234567
-    #    ^  ^     ^       ^^           ^     ^       ^^
+    #    ^ ^    ^       ^^ ^     ^    ^       ^^
     #                                   x
     toks = tokenizer.tokenize(s)
     char_indexes, byte_indexes = tokenizer.calculate_indexes(toks)
-    assert char_indexes == [0, 3, 9, 17, 18, 21, 30, 36, 44, 45]
-    assert byte_indexes == [0, 3, 9, 17, 18, 21, 30, 37, 45, 46]
+    assert char_indexes == [0, 2, 7, 15, 16, 18, 24, 29, 37, 38]
+    assert byte_indexes == [0, 2, 7, 15, 16, 18, 24, 30, 38, 39]
     toks = tokenizer.tokenize(s)
     char_indexes, byte_indexes = tokenizer.calculate_indexes(toks, last_is_end=True)
-    assert char_indexes == [0, 3, 9, 17, 18, 21, 30, 36, 44, 45, 48]
-    assert byte_indexes == [0, 3, 9, 17, 18, 21, 30, 37, 45, 46, 49]
+    assert char_indexes == [0, 2, 7, 15, 16, 18, 24, 29, 37, 38, 40]
+    assert byte_indexes == [0, 2, 7, 15, 16, 18, 24, 30, 38, 39, 41]
 
 
 def test_composite_phrases() -> None:
@@ -491,11 +644,15 @@ def test_lengthening_substitutions() -> None:
     #    ^    ^  ^   ^        ^
     #    x             x
     #             !             lengthening happens here (3ji->þriðji)
-    toks = tokenizer.parse_tokens(s, handle_kludgy_ordinals=tokenizer.KLUDGY_ORDINALS_MODIFY)
+    toks = tokenizer.parse_tokens(
+        s, handle_kludgy_ordinals=tokenizer.KLUDGY_ORDINALS_MODIFY
+    )
     char_indexes, byte_indexes = tokenizer.calculate_indexes(toks)
     assert char_indexes == [0, 5, 8, 12, 21]
     assert byte_indexes == [0, 6, 9, 13, 23]
-    toks = tokenizer.parse_tokens(s, handle_kludgy_ordinals=tokenizer.KLUDGY_ORDINALS_MODIFY)
+    toks = tokenizer.parse_tokens(
+        s, handle_kludgy_ordinals=tokenizer.KLUDGY_ORDINALS_MODIFY
+    )
     char_indexes, byte_indexes = tokenizer.calculate_indexes(toks, last_is_end=True)
     assert char_indexes == [0, 5, 8, 12, 21, 22]
     assert byte_indexes == [0, 6, 9, 13, 23, 24]
@@ -514,3 +671,9 @@ def test_converted_measurements() -> None:
     char_indexes, byte_indexes = tokenizer.calculate_indexes(toks, last_is_end=True)
     assert char_indexes == [0, 7, 14, 16, 22, 26, 29, 34, 41, 42]
     assert byte_indexes == [0, 8, 15, 18, 25, 29, 33, 38, 46, 47]
+
+
+def test_compound() -> None:
+    s = "  Katrín   Júlíusdóttir   var   iðnaðar-  \n\t  og \t\t  viðskiptaráðherra"
+    tlist = list(tokenizer.tokenize(s))
+    assert sum(len(t.original or "") for t in tlist) == len(s)
